; GIMP - The GNU Image Manipulation Program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
;
; 
; Creates animated fire using the steps described in this tutorial by ash44455666
;
; http://www.gimptalk.com/forum/topic/Make-Moving-Fire-20158-1.html#159569
; The animation may be saved with the gif-plug-in.
;
; The script can be found under Filters > Animation > Animated Fire...
;
; The following settings can be modified:
;
; Number of frames (default is 25)
; Image Width and Height (default width is 200 and height is 200)
; Foreground color (default is "burnt" orange)
; Background color (default is black)

;
; 5/3/2008 - added ability to add active layer from existing image to animation and blend
; the layer using the blend mode of the user's choosing.

; 5/6/2008 - added ability to use active layer as a mask to "cut out the flame" to match the mask.
; Just create a black or transparent layer and paint in white the areas you want to show through.
; The mask can work in two ways: the animation is transparent in all areas outside of the mask or
; the areas outside of the mask are completely filled with a color.  In both cases, you need to choose
; the mask background color.  In the case where the areas outside of the mask are transparent, the 
; mask background color will be useful in making the animation blend with your web page's background.
;
; 5/7/2008 - added option to "sandwich" the active layer between the flame layers and set the upper 
; flame layer's blend mode to screen to give the animation the appearance that the active layer is
; actually "in" the fire.  The idea was presented by Darth_Gimp at gimpdome.com.  Also, I modified
; the script to set the "noise" option a little lower when smaller images are created.  With the original
; settings, smaller images had "long" frames that were a little too unrealistic to me.  However, images that
; are much taller than wide may still have this result.
;
; Updated on October 3, 2008 to work in GIMP 2.6
;
; November 20, 2009 - added option to invert the gradient; apply a "cartoon effect" to the flames;
; and the ability to apply several GIMPressionist presets to the flames.  The GIMPressionist option will override
; the cartoon effect.
;
;


; Define the Function

(define (script-fu-fire-anim 	img		 
				drawable
				counter
				width
				height
				useGradient
				gradient
				gradInvert
				foregroundColor
				backgroundColor
				useActive
				blendOpt				
				opacity
				upperLayerX
				upperLayerY
				upperDetail
				lowerLayerX
				lowerLayerY
				lowerDetail
				displaceOpt
				xDisplace
				yDisplace
				blendOpt2
				blendOpt3
				maskColor				
				cartoon
				cartoonAmount
				gimp
				gimpOpt
				)

; Declare the Variables

	(let* (
		    	(theSelection 0)
			(newImage 0)		    
			(activeLayer 0)
			(activeLayerCopy 0)
			(activeLayerHolder 0)
			(floatingSelection 0)
			(activeLayerBlendMode
				(cond 
				(( equal? blendOpt 0 ) NORMAL-MODE)
				(( equal? blendOpt 1 ) DISSOLVE-MODE)
				(( equal? blendOpt 2 ) MULTIPLY-MODE)
				(( equal? blendOpt 3 ) SCREEN-MODE)
				(( equal? blendOpt 4 ) OVERLAY-MODE)
				(( equal? blendOpt 5 ) DIFFERENCE-MODE)				
				(( equal? blendOpt 6 ) ADDITION-MODE)
				(( equal? blendOpt 7 ) SUBTRACT-MODE)
				(( equal? blendOpt 8 ) DARKEN-ONLY-MODE)
				(( equal? blendOpt 9 ) LIGHTEN-ONLY-MODE)
				(( equal? blendOpt 10 ) HUE-MODE)
				(( equal? blendOpt 11 ) SATURATION-MODE)
				(( equal? blendOpt 12 ) COLOR-MODE)
				(( equal? blendOpt 13 ) VALUE-MODE)
				(( equal? blendOpt 14 ) DIVIDE-MODE)
				(( equal? blendOpt 15 ) DODGE-MODE)
				(( equal? blendOpt 16 ) BURN-MODE)
				(( equal? blendOpt 17 ) HARDLIGHT-MODE)
				(( equal? blendOpt 18 ) SOFTLIGHT-MODE)
				(( equal? blendOpt 19 ) GRAIN-EXTRACT-MODE)
				(( equal? blendOpt 20 ) GRAIN-MERGE-MODE)
				)	
			)
			(upperFlameLayerBlendMode
				(cond 
				(( equal? blendOpt2 0 ) DODGE-MODE)
				(( equal? blendOpt2 1 ) DISSOLVE-MODE)
				(( equal? blendOpt2 2 ) MULTIPLY-MODE)
				(( equal? blendOpt2 3 ) SCREEN-MODE)
				(( equal? blendOpt2 4 ) OVERLAY-MODE)
				(( equal? blendOpt2 5 ) DIFFERENCE-MODE)				
				(( equal? blendOpt2 6 ) ADDITION-MODE)
				(( equal? blendOpt2 7 ) SUBTRACT-MODE)
				(( equal? blendOpt2 8 ) DARKEN-ONLY-MODE)
				(( equal? blendOpt2 9 ) LIGHTEN-ONLY-MODE)
				(( equal? blendOpt2 10 ) HUE-MODE)
				(( equal? blendOpt2 11 ) SATURATION-MODE)
				(( equal? blendOpt2 12 ) COLOR-MODE)
				(( equal? blendOpt2 13 ) VALUE-MODE)
				(( equal? blendOpt2 14 ) DIVIDE-MODE)
				(( equal? blendOpt2 15 ) BURN-MODE)
				(( equal? blendOpt2 16 ) HARDLIGHT-MODE)
				(( equal? blendOpt2 17 ) SOFTLIGHT-MODE)
				(( equal? blendOpt2 18 ) GRAIN-EXTRACT-MODE)
				(( equal? blendOpt2 19 ) GRAIN-MERGE-MODE)
				)	
			)
			(lowerFlameLayerBlendMode
				(cond 
				(( equal? blendOpt3 0 ) OVERLAY-MODE)
				(( equal? blendOpt3 1 ) DISSOLVE-MODE)
				(( equal? blendOpt3 2 ) MULTIPLY-MODE)
				(( equal? blendOpt3 3 ) SCREEN-MODE)
				(( equal? blendOpt3 4 ) DIFFERENCE-MODE)				
				(( equal? blendOpt3 5 ) ADDITION-MODE)
				(( equal? blendOpt3 6 ) SUBTRACT-MODE)
				(( equal? blendOpt3 7 ) DARKEN-ONLY-MODE)
				(( equal? blendOpt3 8 ) LIGHTEN-ONLY-MODE)
				(( equal? blendOpt3 9 ) HUE-MODE)
				(( equal? blendOpt3 10 ) SATURATION-MODE)
				(( equal? blendOpt3 11 ) COLOR-MODE)
				(( equal? blendOpt3 12 ) VALUE-MODE)
				(( equal? blendOpt3 13 ) DIVIDE-MODE)
				(( equal? blendOpt3 14 ) DODGE-MODE)
				(( equal? blendOpt3 15 ) BURN-MODE)
				(( equal? blendOpt3 16 ) HARDLIGHT-MODE)
				(( equal? blendOpt3 17 ) SOFTLIGHT-MODE)
				(( equal? blendOpt3 18 ) GRAIN-EXTRACT-MODE)
				(( equal? blendOpt3 19 ) GRAIN-MERGE-MODE)
				)	
			)

			(baseLayer 0)
		    	(baseLayerCopy 0)
			(upperFlame 0)
		    	(lowerFlame 0)
			(upperFlameCopy 0)
			(lowerFlameCopy 0)
			(layerName 0)
		    	(remainingFrames counter)
			(offset 0)
			(step 0)
			(frameNum 1)
            		(borderLayer 0)
			(layerMask 0)
			(preset
				(cond 
				(( equal? gimpOpt 0 ) "Crosshatch")
				(( equal? gimpOpt 1 ) "Dotify")
				(( equal? gimpOpt 2 ) "Embroidery")
				(( equal? gimpOpt 3 ) "Feathers")
				(( equal? gimpOpt 4 ) "Felt-marker")
				(( equal? gimpOpt 5 ) "Mosaic")				
				(( equal? gimpOpt 6 ) "Line-art")
				(( equal? gimpOpt 7 ) "Line-art-2")
				(( equal? gimpOpt 8 ) "Maggot-invasion")
				(( equal? gimpOpt 9 ) "MarbleMadness")
				(( equal? gimpOpt 10 ) "Sample")
				(( equal? gimpOpt 11 ) "Wormcan")
				(( equal? gimpOpt 12 ) "Equalizer")
				(( equal? gimpOpt 13 ) "Equalizer2")
				)	
			)	
	)

(gimp-context-push)
(srand (realtime))

(if (= useActive 0)
	(begin
	(set! newImage (car (gimp-image-new width height RGB)))
	(set! theSelection (car (gimp-selection-save img)))
	(gimp-selection-none img)
	)
)
  
(if (or (= useActive 1) (= useActive 2))
    	(begin
	(set! activeLayer (car (gimp-image-get-active-layer img)))
	(set! theSelection (car (gimp-selection-save img)))
	(gimp-selection-none img)
	(set! width (car (gimp-drawable-width activeLayer)))
	(set! height (car (gimp-drawable-height activeLayer)))
	(set! activeLayer (car (gimp-edit-copy activeLayer)))
	(set! newImage (car (gimp-image-new width height RGB)))
	(set! activeLayerHolder (car (gimp-layer-new newImage width height RGBA-IMAGE 	"Active Layer" 100 NORMAL-MODE)))
	(gimp-drawable-fill activeLayerHolder TRANSPARENT-FILL)	
	(gimp-image-add-layer newImage activeLayerHolder -1)
	(set! floatingSelection (car (gimp-edit-paste activeLayerHolder TRUE))) 	
	(gimp-floating-sel-anchor floatingSelection)	
	(set! activeLayer (car (gimp-image-get-active-layer newImage)))
	)
)

(if (or (= useActive 3) (= useActive 4))
    	(begin
	(set! activeLayer (car (gimp-image-get-active-layer img)))
	(set! theSelection (car (gimp-selection-save img)))
	(gimp-selection-none img)
	(set! width (car (gimp-drawable-width activeLayer)))
	(set! height (car (gimp-drawable-height activeLayer)))
	(set! activeLayer (car (gimp-edit-copy activeLayer)))
	(set! newImage (car (gimp-image-new width height RGB)))
	)
)

(gimp-image-undo-disable newImage)
(set! baseLayer (car (gimp-layer-new newImage width height RGBA-IMAGE "Base Layer" 100 NORMAL-MODE)))
(gimp-image-add-layer newImage baseLayer -1) 
(gimp-context-set-foreground foregroundColor)
(gimp-context-set-background backgroundColor)      
(gimp-edit-blend baseLayer FG-BG-RGB-MODE NORMAL-MODE GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 1 0 TRUE 0 height 0 0)

(set! lowerFlame (car (gimp-layer-copy baseLayer TRUE)))
(gimp-image-add-layer newImage lowerFlame -1)

(if (and (<> upperLayerX 13.0) (< width 400))
    (set! upperLayerX 6.0)
)

(if (and (<> lowerLayerX 13.0) (< width 400))
    (set! lowerLayerX 6.0)
)

(if (= upperDetail -1)
	(set! upperDetail (rand))
)

(if (= lowerDetail -1)
	(set! lowerDetail (rand))
)

(plug-in-solid-noise RUN-NONINTERACTIVE newImage lowerFlame 1 0 (rand) lowerDetail lowerLayerX lowerLayerY)
(set! upperFlame (car (gimp-layer-copy lowerFlame TRUE)))
(gimp-image-add-layer newImage upperFlame -1)
(plug-in-solid-noise RUN-NONINTERACTIVE newImage upperFlame 1 0 (rand) upperDetail upperLayerX upperLayerY)
(set! step (* -1 (/ height counter))) 

(while (> counter 0)
	
(set! baseLayerCopy (car (gimp-layer-copy baseLayer TRUE)))
(set! layerName (string-append "Frame " (number->string frameNum) " (" (number->string 41) "ms)" " (replace)"))
(gimp-image-add-layer newImage baseLayerCopy -1)
(gimp-drawable-set-name baseLayerCopy layerName)
(set! lowerFlameCopy (car (gimp-layer-copy lowerFlame TRUE)))
(gimp-image-add-layer newImage lowerFlameCopy -1)
(gimp-drawable-offset lowerFlameCopy TRUE OFFSET-BACKGROUND 0.0 offset)
(gimp-layer-set-mode lowerFlameCopy lowerFlameLayerBlendMode)
(set! upperFlameCopy (car (gimp-layer-copy upperFlame TRUE)))
(gimp-image-add-layer newImage upperFlameCopy -1)
(gimp-drawable-offset upperFlameCopy TRUE OFFSET-BACKGROUND 0.0 offset)
(gimp-layer-set-mode upperFlameCopy upperFlameLayerBlendMode)

(gimp-image-set-active-layer newImage lowerFlameCopy)
(gimp-image-merge-down newImage lowerFlameCopy CLIP-TO-IMAGE)
(gimp-image-set-active-layer newImage upperFlameCopy)
(gimp-image-merge-down newImage upperFlameCopy CLIP-TO-IMAGE)
(set! baseLayerCopy (car (gimp-image-get-active-layer newImage)))
(if (= useGradient TRUE)
      (begin
      (gimp-context-set-gradient gradient)
	(if (= gradInvert TRUE)
		(begin
		(gimp-invert baseLayerCopy)
		(plug-in-gradmap RUN-NONINTERACTIVE newImage baseLayerCopy)
		)
		(plug-in-gradmap RUN-NONINTERACTIVE newImage baseLayerCopy)
	)
      )
)


(if (= gimp 1)
	(begin
	(plug-in-gimpressionist RUN-NONINTERACTIVE newImage baseLayerCopy preset)
	)
)

(if (= cartoon 1)
	(begin
	(plug-in-emboss RUN-NONINTERACTIVE newImage baseLayerCopy 0.0 90.0 cartoonAmount 0)	
	)
)

(if (= useActive 1)
	(begin
	(set! activeLayerCopy (car (gimp-layer-copy activeLayer TRUE)))
	(gimp-image-add-layer newImage activeLayerCopy -1)
	(gimp-layer-set-mode activeLayerCopy activeLayerBlendMode)
	(gimp-layer-set-opacity activeLayerCopy opacity)
		(if (= displaceOpt TRUE)	
			(plug-in-displace RUN-NONINTERACTIVE newImage activeLayerCopy xDisplace yDisplace TRUE TRUE baseLayerCopy baseLayerCopy 1)	
		)
	(gimp-image-merge-down newImage activeLayerCopy CLIP-TO-IMAGE)
	)
)

(if (= useActive 2)
	(begin
	(set! activeLayerCopy (car (gimp-layer-copy activeLayer TRUE)))
	(gimp-image-add-layer newImage activeLayerCopy -1)
		(if (= displaceOpt TRUE)	
			(plug-in-displace RUN-NONINTERACTIVE newImage activeLayerCopy xDisplace yDisplace TRUE TRUE baseLayerCopy baseLayerCopy 1)	
		)
	(set! baseLayerCopy (car (gimp-layer-copy baseLayerCopy TRUE)))
	(gimp-image-add-layer newImage baseLayerCopy -1)
	(gimp-layer-set-mode baseLayerCopy SCREEN-MODE)
	(gimp-image-set-active-layer newImage activeLayerCopy)
	(gimp-image-merge-down newImage activeLayerCopy CLIP-TO-IMAGE)
	(gimp-image-set-active-layer newImage baseLayerCopy)
	(gimp-image-merge-down newImage baseLayerCopy CLIP-TO-IMAGE)
	)
)	

(if (= useActive 3)
	(begin
	(set! layerMask (car (gimp-layer-create-mask baseLayerCopy ADD-BLACK-MASK)))
	(gimp-layer-add-mask baseLayerCopy layerMask)
	(set! floatingSelection (car (gimp-edit-paste layerMask TRUE)))
	(gimp-floating-sel-anchor floatingSelection)
		(if (= displaceOpt TRUE)	
			(plug-in-displace RUN-NONINTERACTIVE newImage layerMask xDisplace yDisplace TRUE TRUE baseLayerCopy baseLayerCopy 1)	
		)
	(gimp-layer-remove-mask baseLayerCopy MASK-APPLY)
	(gimp-context-set-background maskColor) 
	(plug-in-semiflatten RUN-NONINTERACTIVE newImage baseLayerCopy)
	)
)

(if (= useActive 4)
	(begin
	(set! layerMask (car (gimp-layer-create-mask baseLayerCopy ADD-BLACK-MASK)))
	(gimp-layer-add-mask baseLayerCopy layerMask)
	(set! floatingSelection (car (gimp-edit-paste layerMask TRUE)))
	(gimp-floating-sel-anchor floatingSelection)
		(if (= displaceOpt TRUE)	
			(plug-in-displace RUN-NONINTERACTIVE newImage layerMask xDisplace yDisplace TRUE TRUE baseLayerCopy baseLayerCopy 1)	
		)
	(gimp-layer-remove-mask baseLayerCopy MASK-APPLY)
	(gimp-context-set-background maskColor) 
	(gimp-layer-flatten baseLayerCopy)
	)
)

(set! baseLayerCopy (car (gimp-image-get-active-layer newImage)))

(if (= gimp 2)
	(begin
	(plug-in-gimpressionist RUN-NONINTERACTIVE newImage baseLayerCopy preset)
	)
)

(if (= cartoon 2)
	(begin
	(plug-in-emboss RUN-NONINTERACTIVE newImage baseLayerCopy 0.0 90.0 cartoonAmount 0)	
	)
)


(set! offset (+ offset step))
(set! frameNum (+ frameNum 1))
(set! counter (- counter 1))




) ; goes with while

; Removes the original layer from the stack because it's no longer needed  
  
(gimp-image-remove-layer newImage baseLayer) 
(gimp-image-remove-layer newImage upperFlame) 
(gimp-image-remove-layer newImage lowerFlame) 

(if (or (= useActive 1) (= useActive 2))
	(gimp-image-remove-layer newImage activeLayer)
)

(gimp-image-undo-enable newImage)

(gimp-selection-load theSelection)
(gimp-image-remove-channel img theSelection)
(gimp-display-new newImage)
(gimp-image-set-active-layer img drawable)
(gimp-context-pop)

(gimp-displays-flush)

; Displays the final animation


  )
)

(script-fu-register "script-fu-fire-anim"
  "<Image>/Filters/Animation/Animators/Animated Fire..."
  "Creates an animated fire effect"
  "Art Wade"
  "Art Wade"
  "March 10, 2010"
  "RGB*"
  SF-IMAGE       	"Image" 0
  SF-DRAWABLE    	"Drawable" 0
  SF-ADJUSTMENT 	"Number of frames" '(25 1 500 1 10 0 1)
  SF-ADJUSTMENT 	"Image Width" '(200 50 2500 1 1 0 1)
  SF-ADJUSTMENT 	"Image Height" '(200 50 2500 1 1 0 1)
  SF-TOGGLE       "Use Gradient?" TRUE
  SF-GRADIENT     "Gradient" 		"Incandescent"
  SF-TOGGLE       "Invert Gradient?" FALSE
  SF-COLOR      	"Desired Flame Foreground Color (Only used if gradient is not applied)" '(255 132 0)
  SF-COLOR	  	"Desired Flame Background Color (Only used if gradient is not applied)" '(0 0 0)
  SF-OPTION		"Add Active Layer to Animation?" 	'("Don't use active layer in animation"
									"Above animation using blend mode below"
									"Place render between two fire layers and set upper fire layer mode to Screen"
									"As a mask; transparency around mask; use mask background color to blend edges"
									"As a mask; fill image with mask background color"
									)
  SF-OPTION		"Blend Mode for Active Layer" 		'("NORMAL-MODE" 
									"DISSOLVE-MODE"
									"MULTIPLY-MODE"
									"SCREEN-MODE"
									"OVERLAY-MODE"
									"DIFFERENCE-MODE"
									"ADDITION-MODE"
									"SUBTRACT-MODE"
									"DARKEN-ONLY-MODE"
									"LIGHTEN-ONLY-MODE"
									"HUE-MODE"
									"SATURATION-MODE"
									"COLOR-MODE"
									"VALUE-MODE"
									"DIVIDE-MODE"
									"DODGE-MODE"
									"BURN-MODE"
									"HARDLIGHT-MODE"
									"SOFTLIGHT-MODE"
									"GRAIN-EXTRACT-MODE"
									"GRAIN-MERGE-MODE")
  SF-ADJUSTMENT   "Opacity for Active Layer" '(100.0 0.0 100.0 1 1 1 0)
  SF-ADJUSTMENT   "Upper Layer X Size Noise" '(13.0 0.1 16.0 1 1 1 1)
  SF-ADJUSTMENT   "Upper Layer Y Size Noise" '(2.0 0.1 16.0 1 1 1 1)
  SF-ADJUSTMENT   "Upper Layer Noise Detail (-1 is random)" '(-1 -1 15 1 1 0 1)  
  SF-ADJUSTMENT   "Lower Layer X Size Noise" '(13.0 0.1 16.0 1 1 1 1)
  SF-ADJUSTMENT   "Lower Layer Y Size Noise" '(5.3 0.1 16.0 1 1 1 1)
  SF-ADJUSTMENT   "Lower Layer Noise Detail (-1 is random)" '(-1 -1 15 1 1 0 1)  
  SF-TOGGLE       "Move active layer with flames?" TRUE
  SF-ADJUSTMENT   "Move amount along x-axis" '(5 -9999 9999 1 1 0 1)
  SF-ADJUSTMENT   "Move amount along y-axis" '(5 -9999 9999 1 1 0 1)  
  SF-OPTION		"Upper Flame Layer Blend Mode" 		'("DODGE-MODE" 
									"DISSOLVE-MODE"
									"MULTIPLY-MODE"
									"SCREEN-MODE"
									"OVERLAY-MODE"
									"DIFFERENCE-MODE"
									"ADDITION-MODE"
									"SUBTRACT-MODE"
									"DARKEN-ONLY-MODE"
									"LIGHTEN-ONLY-MODE"
									"HUE-MODE"
									"SATURATION-MODE"
									"COLOR-MODE"
									"VALUE-MODE"
									"DIVIDE-MODE"
									"BURN-MODE"
									"HARDLIGHT-MODE"
									"SOFTLIGHT-MODE"
									"GRAIN-EXTRACT-MODE"
									"GRAIN-MERGE-MODE")
  SF-OPTION		"Lower Flame Layer Blend Mode" 		'("OVERLAY-MODE" 
									"DISSOLVE-MODE"
									"MULTIPLY-MODE"
									"SCREEN-MODE"
									"DIFFERENCE-MODE"
									"ADDITION-MODE"
									"SUBTRACT-MODE"
									"DARKEN-ONLY-MODE"
									"LIGHTEN-ONLY-MODE"
									"HUE-MODE"
									"SATURATION-MODE"
									"COLOR-MODE"
									"VALUE-MODE"
									"DIVIDE-MODE"
									"DODGE-MODE"
									"BURN-MODE"
									"HARDLIGHT-MODE"
									"SOFTLIGHT-MODE"
									"GRAIN-EXTRACT-MODE"
									"GRAIN-MERGE-MODE")

  SF-COLOR      	"Mask Background Color" 		'(0 0 0)
  SF-OPTION		"Cartoonize?" 		'("No"
						  "Flames Only"
						  "Final Image") 
  SF-ADJUSTMENT 	"How much cartooning?" '(5 3 20 1 1 0 1)
  SF-OPTION		"Add GIMPressionist?" 		'("No"
						  "Flames Only"
						  "Final Image")
  SF-OPTION		"GIMPressionist Preset" 		'("Crosshatch" 
									"Dotify"
									"Embroidery"
									"Feathers"
									"Felt-marker"
									"Mosaic"
									"Line-art"
									"Line-art-2"
									"Maggot-invasion"
									"MarbleMadness"
									"Sample"
									"Wormcan"
									"Equalizer"
									"Equalizer 2"	)
)


                         
